'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.runHandlers = exports.addHandler = exports.clearContext = undefined;

var _util = require('./util');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var contexts = {};

// Singleton context manager.
//
// This object manages component removal handlers for all components beloning to
// one or more redux store instances. Each redux store is populated with its own
// context id value, under which its unique handlers are stored here.
//
// This allows components with identical id values to register handlers within
// the same runtime without conflict. By keeping track of which redux store
// instance each handler belongs to, we can ensure that each user context
// remains completely isolated in a shared runtime such as the server.

/**
 * Remove an entire context an all of its removal handlers. This is called when
 * the `endContext` action is dispatched.
 *
 * @param {String} contextId The id of a context.
 * @returns {undefined}
 */
var clearContext = exports.clearContext = function clearContext(contextId) {
  contexts = (0, _util.omit)(contexts, contextId);
};

/**
 * Register a removal handler for a component id within a context. Any number of
 * handlers can be registerd for a single id.
 *
 * @param {String} contextId The id of a context.
 * @param {String} componentId The id of a component.
 * @param {Function} handler A removal handler function.
 * @returns {undefined}
 */
var addHandler = exports.addHandler = function addHandler(contextId, componentId, handler) {
  // Ensure context exists for the provided id.
  var context = contexts[contextId] = contexts[contextId] || {};

  // Assingn a new array with the provided handler or add it to an existing
  // array of handlers.
  context[componentId] = context[componentId] ? [].concat(_toConsumableArray(context[componentId]), [handler]) : [handler];
};

/**
 * Invoke the removal handlers for a component id within a context. Each handler
 * is called with an `event` object with an interface that mimics DOM events.
 * The removal of the component can be prevented by calling `preventDefault()`
 * on the event object from within a handler. A provided relocation component
 * object containing `id`, `props`, and `type` is assigned as the event target.
 *
 * @param {String} contextId The id of a context.
 * @param {Object} component A Relocation component object.
 * @returns {undefined}
 */
var runHandlers = exports.runHandlers = function runHandlers(contextId, component) {
  // Get the handlers for the current component it in the current context
  // if any were registered.
  var handlers = contexts[contextId] && contexts[contextId][component.id];

  if (!handlers) {
    // No handlers were registered, return `true` to proceed with component
    // removal.
    return true;
  }

  var event = {
    target: component,
    defaultPrevented: false,
    preventDefault: function preventDefault() {
      event.defaultPrevented = true;
    }
  };

  for (var i = 0, len = handlers.length; i < len; i++) {
    handlers[i](event);
  }

  if (event.defaultPrevented) {
    // `event.preventDefault` was called. Return `false` to cancel component
    // removal.
    return false;
  }

  // Remove handlers for this component from the context.
  contexts[contextId] = (0, _util.omit)(contexts[contextId], component.id);

  // If there are no handlers left for the context, remove it.
  contexts = !Object.keys(contexts[contextId]).length ? (0, _util.omit)(contexts, contextId) : contexts;

  // Return `true` to proceed with component removal.
  return true;
};
//# sourceMappingURL=manager.js.map